/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package chat.gui;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Chat CUI
 */
public class ChatGUIImpl extends AbstractComponent implements DisplayMessage
{

  JFrame         chatFrame;
  JScrollPane    messagesScroll;
  JTextArea      messagesArea;
  JTextField     messageField;
  JButton        sendMessageButton;
  ActionListener publishMessageListener;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  PublishMessage publishMessageItf;

  // ---------------------------------------------------------------------------
  // Implementation of the DisplayMessage interface
  // ---------------------------------------------------------------------------

  /**
   * @see DisplayMessage#displayMessage(String, String)
   */
  public void displayMessage(String from, String message)
  {
    getMessagesArea().append(from + " : " + message + "\n");
    getMessagesArea().scrollRectToVisible(
        new Rectangle(0, getMessagesArea().getHeight(), 1, 1));
  }

  // ---------------------------------------------------------------------------
  // GUI initialization methods
  // ---------------------------------------------------------------------------

  /**
   * For test only
   * 
   * @param args ignored.
   */
  public static void main(String[] args)
  {
    ChatGUIImpl gui = new ChatGUIImpl();
    gui.getChatFrame().setVisible(true);
    gui.getChatFrame().setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    gui.displayMessage("toto", "tu es gros !");
    gui.displayMessage("titi", "Qui est gros ?");
  }

  private JFrame getChatFrame()
  {
    if (chatFrame == null)
    {
      chatFrame = new JFrame("Dream chat");
      chatFrame.setSize(300, 300);
      chatFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      Container pane = chatFrame.getContentPane();
      GridBagLayout layout = new GridBagLayout();
      GridBagConstraints c = new GridBagConstraints();
      pane.setLayout(layout);
      c.fill = GridBagConstraints.BOTH;
      c.insets = new Insets(10, 10, 5, 10);
      c.gridx = 0;
      c.gridy = 0;
      c.gridwidth = 2;
      c.gridheight = 1;
      c.weightx = 1;
      c.weighty = 1;
      layout.setConstraints(getMessagesScroll(), c);

      c.insets = new Insets(5, 10, 10, 5);
      c.gridx = 0;
      c.gridy = 1;
      c.gridwidth = 1;
      c.gridheight = 1;
      c.weightx = 1;
      c.weighty = 0;
      layout.setConstraints(getMessageField(), c);

      c.insets = new Insets(5, 5, 10, 10);
      c.gridx = 1;
      c.gridy = 1;
      c.gridwidth = 1;
      c.gridheight = 1;
      c.weightx = 0;
      c.weighty = 0;
      layout.setConstraints(getSendMessageButton(), c);

      pane.add(getMessagesScroll());
      pane.add(getMessageField());
      pane.add(getSendMessageButton());
    }
    return chatFrame;
  }

  private JTextField getMessageField()
  {
    if (messageField == null)
    {
      messageField = new JTextField();
      messageField.addActionListener(getPublishMessageListener());
    }
    return messageField;
  }

  private JScrollPane getMessagesScroll()
  {
    if (messagesScroll == null)
    {
      messagesScroll = new JScrollPane();
      messagesScroll.setViewportView(getMessagesArea());
    }
    return messagesScroll;
  }

  private JTextArea getMessagesArea()
  {
    if (messagesArea == null)
    {
      messagesArea = new JTextArea();
      messagesArea.setEditable(false);
    }
    return messagesArea;
  }

  private JButton getSendMessageButton()
  {
    if (sendMessageButton == null)
    {
      sendMessageButton = new JButton();
      sendMessageButton.setText("Send");
      sendMessageButton.addActionListener(getPublishMessageListener());
    }
    return sendMessageButton;
  }

  private ActionListener getPublishMessageListener()
  {
    if (publishMessageListener == null)
    {
      publishMessageListener = new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          String msg = getMessageField().getText();
          getMessageField().setText(null);
          publishMessageItf.publishMessage(msg);
        }
      };
    }
    return publishMessageListener;
  }

  // ---------------------------------------------------------------------------
  // Overridden methods
  // ---------------------------------------------------------------------------

  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    getChatFrame().setVisible(true);
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(PublishMessage.ITF_NAME))
    {
      publishMessageItf = (PublishMessage) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{PublishMessage.ITF_NAME};
  }
}
