/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package helloworld;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.manager.TaskManager;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/** Producer component implementation. */
public class ProducerImpl extends AbstractComponent
{
  private ChunkFactoryReference<HelloWorldChunk> helloworldChunkFactory;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  // the Push interface.
  private Push                                   outPushItf;
  // the message manager interface.
  private MessageManager                         messageManagerItf;

  // ---------------------------------------------------------------------------
  // Overridden methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractComponent#beforeFirstStart(Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      Map<String, Object> hints = new HashMap<String, Object>();
      hints.put("period", new Long(1000));
      Util.addTask(componentItf, new ClientTask("Hello World task1"), hints);
      logger.log(BasicLevel.DEBUG, "tasks added");
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME,
        TaskManager.ITF_NAME};
  }

  private class ClientTask extends AbstractTask
  {

    private String message;

    /**
     * Constructor
     */
    public ClientTask(String message)
    {
      super(message);
      this.message = message;
    }

    int seqno = 0;

    /**
     * @see org.objectweb.dream.control.activity.task.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      try
      {
        Message msg = messageManagerItf.createMessage();
        ChunkFactoryReference<HelloWorldChunk> helloworldChunkFactory = messageManagerItf
            .getChunkFactory(HelloWorldChunk.class);
        HelloWorldChunk chunk = messageManagerItf
            .createChunk(helloworldChunkFactory);
        chunk.setMessage(message + " seqno=" + (seqno++));
        messageManagerItf.addChunk(msg, HelloWorldChunk.DEFAULT_NAME, chunk);
        outPushItf.push(msg);
      }
      catch (Exception e)
      {
        e.printStackTrace();
        return STOP_EXECUTING;
      }
      return EXECUTE_AGAIN;
    }
  }
}
