/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet 
 */

package org.objectweb.dream.aggregator;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * This aggregator aggregates a fixed number of messages. These messages are
 * collected on its Pull input. The number of messages to be aggregated is set
 * using the {@link PullPullAggregatorFixedNumberAttributeController} interface.
 */
public class PullPullAggregatorFixedNumberImpl extends AbstractComponent
    implements
      Pull,
      PullPullAggregatorFixedNumberAttributeController
{

  private Pull             inPullItf;

  /** the message manager client interface of this component */
  protected MessageManager messageManagerItf;

  private int              nbMessageToAggregate;

  // ---------------------------------------------------------------------------
  // Implementation of the Pull interface.
  // ---------------------------------------------------------------------------
  /**
   * @see Pull#pull()
   */
  public Message pull() throws PullException
  {
    Message outMessage = messageManagerItf.createMessage();
    for (int i = 0; i < nbMessageToAggregate; i++)
    {
      messageManagerItf.addSubMessage(outMessage, inPullItf.pull());
    }
    return outMessage;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the PullPullAggregatorFixedNumberAttributeController
  // interface.
  // ---------------------------------------------------------------------------

  /**
   * @see PullPullAggregatorFixedNumberAttributeController#getNbMessagesToAggregate()
   */
  public int getNbMessagesToAggregate()
  {
    return nbMessageToAggregate;
  }

  /**
   * @see PullPullAggregatorFixedNumberAttributeController#setNbMessagesToAggregate(int)
   */
  public void setNbMessagesToAggregate(int nb)
  {
    this.nbMessageToAggregate = nb;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Pull.IN_PULL_ITF_NAME))
    {
      inPullItf = (Pull) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Pull.IN_PULL_ITF_NAME, MessageManager.ITF_NAME};
  }

}