/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.aggregator;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Attribute controller interface for
 * {@link PullPullAggregatorWhileNotNullImpl }aggregator implementation.
 * Defines the behavior when the first pulled message is null.
 */
public interface PullPullAggregatorWhileNotNullAttributeController
    extends
      AttributeController
{
  /**
   * Returns <code>true</code> if the aggregator must return an empty message
   * when no message can't be aggregated. If <code>false</code>, the
   * aggregator returns <code>null</code>.
   * 
   * @return the current <code>pullEmptyMessage</code> policy.
   */
  boolean getPullEmptyMessagePolicy();

  /**
   * Sets the <code>pullEmptyMessage</code> policy.
   * 
   * @param policy the new <code>pullEmptyMessage</code> policy.
   * @see #getPullEmptyMessagePolicy()
   */
  void setPullEmptyMessagePolicy(boolean policy);
}