/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.protocol.ExportException;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.InvalidExportIdentifierException;
import org.objectweb.dream.protocol.Protocol;
import org.objectweb.dream.protocol.channel.ChannelProtocol;
import org.objectweb.dream.protocol.messagePassing.MessagePassingOutgoingPush;
import org.objectweb.dream.protocol.messagePassing.MessagePassingProtocol;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * ChannelIn implementation.
 */
public class ChannelOutImpl extends AbstractComponent
    implements
      Push,
      ChannelInExportId
{
  private MessagePassingOutgoingPush lowerOutgoingPush;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  private MessagePassingProtocol     lowerProtocolItf;
  private DestinationResolver        destinationResolverItf;

  // ---------------------------------------------------------------------------
  // Constructor
  // ---------------------------------------------------------------------------

  /**
   * Default constructor.
   */
  public ChannelOutImpl()
  {
    fcNeedAsyncStart = true;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    try
    {
      lowerOutgoingPush.outgoingPush(message, destinationResolverItf
          .resolveDestination(message));
    }
    catch (InvalidExportIdentifierException e)
    {
      throw new PushException("Invalid destination for message", e);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the ChannelInExportId interface
  // ---------------------------------------------------------------------------

  /**
   * @see ChannelInExportId#getExportIdentifier()
   */
  public ExportIdentifier getExportIdentifier()
  {
    return lowerOutgoingPush.getLocalExportIdentifier();
  }

  // ---------------------------------------------------------------------------
  // Overridden methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractComponent#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    super.startFc();
    try
    {
      lowerOutgoingPush = lowerProtocolItf.export(null, null);
    }
    catch (ExportException e)
    {
      throw new IllegalLifeCycleException("Unable to export access point : "
          + e.getMessage());
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Protocol.LOWER_PROTOCOL_ITF_NAME,
        DestinationResolver.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(ChannelProtocol.LOWER_PROTOCOL_ITF_NAME))
    {
      lowerProtocolItf = (MessagePassingProtocol) serverItf;
    }
    else if (clientItfName.equals(DestinationResolver.ITF_NAME))
    {
      destinationResolverItf = (DestinationResolver) serverItf;
    }
  }
}
