/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * Utility methods.
 */
public final class Util
{
  private Util()
  {
  }

  /**
   * Reads and returns an array of export identifier from an
   * <code>ObjectInput</code> object.
   * 
   * @param in the input from which to read.
   * @return an array of export identifier.
   * @throws IOException if an error occurs.
   * @throws ClassNotFoundException if the class of read objects is not found.
   */
  public static ExportIdentifier[] readExportIdentifierArray(ObjectInput in)
      throws IOException, ClassNotFoundException
  {
    int length = in.readInt();
    if (length == 0)
    {
      return ExportIdentifier.EMPTY_EXPORT_IDENTIFIER_ARRAY;
    }
    ExportIdentifier[] toReturn = new ExportIdentifier[length];
    for (int i = 0; i < length; i++)
    {
      toReturn[i] = (ExportIdentifier) org.objectweb.dream.util.Util
          .readObject(in);
    }
    return toReturn;
  }

  /**
   * Writes an array of export identifier on an <code>ObjectOutput</code>
   * object.
   * 
   * @param out the output on which to write.
   * @param array the array of export identifier to write (cannot be
   *          <code>null</code>).
   * @throws IOException if an error occurs.
   */
  public static void writeExportIdentifierArray(ObjectOutput out,
      ExportIdentifier[] array) throws IOException
  {
    int length = array.length;
    out.writeInt(length);
    for (int i = 0; i < length; i++)
    {
      org.objectweb.dream.util.Util.writeObject(out, array[i]);
    }

  }
}
