/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.bus.logicalClock;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.util.Util;

/**
 * Export identifier handled by logical clock protocol.
 */
public class LogicalClockExportIdentifier
    implements
      ExportIdentifier,
      Externalizable
{
  private static final long  serialVersionUID              = 1L;
  /** The type value corresponding to a FIFO clock. */
  public static final int    FIFO_CLOCK_TYPE               = 1;
  /** The name of the type corresponding to a FIFO clock. */
  public static final String FIFO_CLOCK_TYPE_NAME          = "fifo";
  /** The type value corresponding to a matrix causal clock. */
  public static final int    MATRIX_CAUSAL_CLOCK_TYPE      = 2;
  /** the name of the type corresponding to a matrix causal clock. */
  public static final String MATRIX_CAUSAL_CLOCK_TYPE_NAME = "matrix-causal";

  private int                clockType;
  private String             chunkName;
  private ExportIdentifier[] nexts                         = new ExportIdentifier[1];

  /**
   * @param type the logical clock type either {@link #FIFO_CLOCK_TYPE} or
   *          {@link #MATRIX_CAUSAL_CLOCK_TYPE}.
   * @param chunkName the name of the {@link LogicalClockChunk}.
   * @param next the export identifier of the lower level protocol.
   */
  public LogicalClockExportIdentifier(int type, String chunkName,
      ExportIdentifier next)
  {
    clockType = type;
    this.chunkName = chunkName;
    nexts[0] = next;
  }

  /**
   * @return Returns the clockType.
   */
  public int getClockType()
  {
    return clockType;
  }

  /**
   * @return Returns the chunkName.
   */
  public String getChunkName()
  {
    return chunkName;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the ExportIdentifier interface
  // ---------------------------------------------------------------------------

  /**
   * @see ExportIdentifier#getNextExportIds()
   */
  public ExportIdentifier[] getNextExportIds()
  {
    return nexts;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Externalizable interface
  // ---------------------------------------------------------------------------

  /**
   * @see Externalizable#readExternal(ObjectInput)
   */
  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    clockType = in.readInt();
    chunkName = in.readUTF();
    nexts[0] = (ExportIdentifier) Util.readObject(in);
  }

  /**
   * @see Externalizable#writeExternal(ObjectOutput)
   */
  public void writeExternal(ObjectOutput out) throws IOException
  {
    out.writeInt(clockType);
    out.writeUTF(chunkName);
    Util.writeObject(out, nexts[0]);
  }

}
