/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.rpc;

import java.util.Map;

import org.objectweb.dream.protocol.BindException;
import org.objectweb.dream.protocol.ExportException;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.InvalidExportIdentifierException;
import org.objectweb.dream.protocol.Protocol;

/**
 * RPC Protocol. very early version !!!
 */
public interface RPCProtocol extends Protocol
{
  /**
   * Exports a {@link Skeleton} interface using this protocol graph. This method
   * informs the protocol that the provided <code>Skeleton</code> is willing
   * to accept connection. The returned export identifier is a specific name for
   * the exported channel.
   * 
   * @param skeleton a <code>Skeleton</code> interface to export.
   * @param hints additional parameters, can be <code>null</code>.
   * @return an identifier for the exported interface.
   * @throws ExportException if the export process fails.
   */
  ExportIdentifier export(Skeleton skeleton, Map<String, Object> hints) throws ExportException;

  /**
   * Binds to a previously exported {@link Skeleton} interface designated by the
   * given export identifier. This methods instantiate a new communication
   * channel and returns a stub interface on which remote method invocations can
   * be performed.
   * 
   * @param exportId the identifier of the exported interface.
   * @param hints additional parameters, can be <code>null</code>.
   * @return a <code>stub</code> interface on which remote invocation can be
   *         performed.
   * @throws InvalidExportIdentifierException if the given export identifier is
   *           incorrect.
   * @throws BindException if the binding process fails.
   */
  Stub bind(ExportIdentifier exportId, Map<String, Object> hints)
      throws InvalidExportIdentifierException, BindException;
}
