/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.pump;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Attribute controller for pump components. It allows defining three policies:
 * <ul>
 * <li>the <code>PushNullPolicy</code> indicates whether "null" messages that
 * are pulled must be pushed or not. Its default value is <code>false</code>.
 * </li>
 * <li>the <code>StopOnPushExceptionPolicy</code> indicates whether the task
 * must be stopped when a {@link org.objectweb.dream.PushException} occurs. Its
 * default value is <code>true</code>.</li>
 * <li>the <code>StopOnPullExceptionPolicy</code> indicates whether the task
 * must be stopped when a {@link org.objectweb.dream.PullException} occurs. Its
 * default value is <code>true</code>.</li>
 * </ul>
 */
public interface PumpAttributeController extends AttributeController
{

  /**
   * Returns the PushNullPolicy.
   * 
   * @return the PushNullPolicy.
   */
  boolean getPushNullPolicy();

  /**
   * Sets the PushNullPolicy.
   * 
   * @param pushNull the PushNullPolicy.
   */
  void setPushNullPolicy(boolean pushNull);

  /**
   * Returns the StopOnPushExceptionPolicy.
   * 
   * @return the StopOnPushExceptionPolicy.
   */
  boolean getStopOnPushExceptionPolicy();

  /**
   * Sets the StopOnPushExceptionPolicy.
   * 
   * @param stopOnPushException the StopOnPushExceptionPolicy.
   */
  void setStopOnPushExceptionPolicy(boolean stopOnPushException);

  /**
   * Returns the StopOnPullExceptionPolicy.
   * 
   * @return the StopOnPullExceptionPolicy.
   */
  boolean getStopOnPullExceptionPolicy();

  /**
   * Sets the StopOnPullExceptionPolicy.
   * 
   * @param stopOnPullException the StopOnPullExceptionPolicy.
   */
  void setStopOnPullExceptionPolicy(boolean stopOnPullException);
}
