/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.pushwithreturn;

import org.objectweb.dream.message.Message;

/**
 * This interface defines a key generator. Such a generator allows generating
 * keys for identifying messages.
 * <p>
 * Note that a key generator may change the message: it can, for instance,
 * insert a chunk that contains the key.
 * 
 * @see Key
 */
public interface KeyGenerator
{
  /** The commonly used name to refer to this interface. */
  String ITF_NAME = "key-generator";

  /**
   * Generates a key for the given message. This method possibly adds a
   * <code>KeyChunk</code> to the message.
   * 
   * @param message the message for which a key must be generated.
   * @return the generated key.
   * @throws Exception if this is not possible to generate a key (for instance
   *           when the generator try to add a Chunk to a non extensible
   *           message).
   */
  Key generateKey(Message message) throws Exception;

}