/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.message.Message;

/**
 * Calculates keys for messages. Keys are objects that can be used to index
 * messages in a buffer.
 */
public interface MessageKeyManager
{

  /**
   * The commonly used name to refer to an interface with this signature.
   */
  String ITF_NAME = "message-key-manager";

  /**
   * Returns a new empty key object.
   * 
   * @return a new empty key object; never <code>null</code>.
   */
  Object createKey();

  /**
   * Fills the given key to match the given message.
   * 
   * @param key the key to calculate; must not be <code>null</code>.
   * @param message the message to which the key will be associated; must not be
   *          <code>null</code>
   */
  void fillKey(Object key, Message message);

  /**
   * Clones the given key.
   * 
   * @param key the key to clone; never <code>null</code>.
   * @return a clone of the given key; never <code>null</code>.
   */
  Object duplicateKey(Object key);

  /**
   * Delete the given key instance.
   * 
   * @param key the key to delete; never <code>null</code>.
   */
  void deleteKey(Object key);

}