/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.serializator;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.codec.CodecInputOutput;
import org.objectweb.dream.message.codec.MessageCodec;
import org.objectweb.dream.pool.ObjectPool;
import org.objectweb.dream.pool.Recyclable;
import org.objectweb.dream.pool.RecyclableFactory;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Basic implementation of the {@link DeSerializator}interface.
 */
public class DeSerializatorImpl extends AbstractComponent
    implements
      DeSerializator,
      RecyclableFactory
{

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected MessageCodec messageCodecItf;
  protected ObjectPool   objectPoolItf;

  // ---------------------------------------------------------------------------
  // Implementation of the Serializator interface
  // ---------------------------------------------------------------------------

  /**
   * @see DeSerializator#deserialize(byte[])
   */
  public Message deserialize(byte[] byteArray) throws IOException
  {
    CodecInputOutput codecInputOutput = (CodecInputOutput) objectPoolItf
        .newInstance();
    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
        byteArray);
    codecInputOutput.setInput(byteArrayInputStream);
    logger.log(BasicLevel.DEBUG, "read message");
    Message message = messageCodecItf.decode(codecInputOutput);
    objectPoolItf.recycleInstance((Recyclable) codecInputOutput);
    return message;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the RecyclableFactory interface
  // ---------------------------------------------------------------------------

  /**
   * @see RecyclableFactory#newInstance()
   */
  public Recyclable newInstance()
  {
    return new CodecInputOutputByteArray();
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageCodec.ITF_NAME))
    {
      messageCodecItf = (MessageCodec) serverItf;
    }
    else if (clientItfName.equals(ObjectPool.ITF_NAME))
    {
      objectPoolItf = (ObjectPool) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageCodec.ITF_NAME, ObjectPool.ITF_NAME};
  }

}
