/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.synchro;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.InterruptedPullException;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * This component has a Pull output. When its pull method is called, it locks a
 * mutex (to which it is bound), calls pulls a message on its input, and then
 * unlocks the mutex.
 */
public class LockPullUnlockImpl extends AbstractComponent implements Pull
{

  protected Pull  inPullItf;
  protected Mutex mutexItf;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Pull#pull()
   */
  public Message pull() throws PullException
  {
    Message msg = null;
    try
    {
      mutexItf.lock();
    }
    catch (InterruptedException e)
    {
      throw new InterruptedPullException(e);
    }
    try
    {
      msg = inPullItf.pull();
    }
    finally
    {
      mutexItf.unlock();
    }
    return msg;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Pull.IN_PULL_ITF_NAME))
    {
      inPullItf = (Pull) serverItf;
    }
    else if (clientItfName.equals(Mutex.ITF_NAME))
    {
      mutexItf = (Mutex) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Pull.IN_PULL_ITF_NAME, Mutex.ITF_NAME};
  }

}