/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq, Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.synchro;

/**
 * A mutex component aims at synchronizing the execution of several components.
 */
public interface Mutex
{

  /** The default name of the {@link Mutex }interface. */
  String ITF_NAME = "mutex";

  /**
   * Acquires the mutex.
   * 
   * @throws InterruptedException the thread is interrupted.
   */
  void lock() throws InterruptedException;

  /**
   * Tries to acquire the mutex.
   * 
   * @param msecs the number of milliseconds to wait for the lock.
   * @return <code>true</code> if the lock is acquired, <code>false</code>
   *         otherwise.
   * @throws InterruptedException if the thread is interrupted.
   */
  boolean timedLock(long msecs) throws InterruptedException;

  /**
   * Releases the mutex.
   */
  void unlock();
}