/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.synchro;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * An attribute controller allowing setting the initial number of permits of the
 * semaphore.
 */
public interface SemaphoreAttributeController extends AttributeController
{

  /**
   * Sets the initial number of permits of the semaphore. Setting
   * <code>initialPermits</code> to one makes the semaphore act as a mutual
   * exclusion lock. Negative seeds are also allowed, in which case no acquires
   * will proceed until the number of releases has pushed the number of permits
   * past 0.
   * 
   * @param initialPermits the initial number of permits to set.
   */
  void setInitialPermits(long initialPermits);

  /**
   * Returns the initial number of permits of the semaphore.
   * 
   * @return the initial number of permits of the semaphore.
   */
  long getInitialPermits();

}